package JIP;

import Ugosoft.awt.XFrame;
import Ugosoft.io.*;
import java.awt.*;
import java.io.*;
import java.net.*;
import java.awt.event.*;
import JIP.engine.*;
    
class EditFrame extends XFrame implements ActionListener, KeyListener
{
    private TextArea  m_editArea;
    private TextArea  m_consultArea;
    private JIPEngine m_engine;
    private String    m_strWndID;
    private String    m_strFileName;
   
    private static int    s_nDocument = 0;
    
    public EditFrame(JIPEngine engine, DataInputStream ins)
    {
        this(engine, false);
                
        loadFile(ins);
        
		consultEditArea();
    }
    
    public EditFrame(JIPEngine engine, boolean bOpen)
    {
        super();     
        s_nDocument++;
        setTitle("New program " + Integer.toString(s_nDocument));
        URL url = getClass().getResource("/icoJipDoc.gif");
        if(url != null)
            setIconImage(Toolkit.getDefaultToolkit().getImage(url));
        
        Dimension ds = Toolkit.getDefaultToolkit().getScreenSize();
        setSize((int) ((double)ds.width * (6.0/10.0)), (int)((double)ds.height * (7.0/10.0)));
        
        m_strWndID = "JIP" + Integer.toString(s_nDocument);
        m_engine = engine;                
        
        MenuBar menuBar = new MenuBar();                        
        Menu menu = new Menu("File");
        MenuItem mi;

        mi = new MenuItem("New");
        mi.addActionListener(this); 
        menu.add(mi);

        //MenuShortcut mnsh = new MenuShortcut('o');
        //mi = new MenuItem("Open", mnsh);
        mi = new MenuItem("Open");
        mi.addActionListener(this); 
        menu.add(mi);

        ////////////////
        
        //mnsh = new MenuShortcut('s');
        //mi = new MenuItem("Save", mnsh);
        mi = new MenuItem("Save");
        mi.addActionListener(this); // e' necessario per ogni item nel menu
        menu.add(mi);
        
        ////////////////
        
        //mnsh = new MenuShortcut('a');
        //mi = new MenuItem("Save as", mnsh);
        mi = new MenuItem("Save as");
        mi.addActionListener(this); // e' necessario per ogni item nel menu
        menu.add(mi);
        
        /////////////////
        
        menu.addSeparator();
        
        /////////////////
        
        //mnsh = new MenuShortcut('n');
        //mi = new MenuItem("Consult", mnsh);
        mi = new MenuItem("Consult");
        mi.addActionListener(this); // e' necessario per ogni item nel menu
        menu.add(mi);
        
        /////////////////
        
        menu.addSeparator();
        
        /////////////////
        
        //mnsh = new MenuShortcut('l');
        //mi = new MenuItem("Close", mnsh);
        mi = new MenuItem("Close");
        mi.addActionListener(this); // e' necessario per ogni item nel menu
        menu.add(mi);
                
        /////////////////
        
        menuBar.add(menu);                       
        
        /////////////////

        menu = new Menu("Help");
        //mnsh = new MenuShortcut('h');
        //mi = new MenuItem("Help me", mnsh);
        mi = new MenuItem("Help me");
        mi.addActionListener(this); 
        menu.add(mi);
        
        /////////////////
                       
        menuBar.add(menu);        
        setMenuBar(menuBar);
        
        /////////////////
                
        setLayout(new BorderLayout());
        
        m_editArea    = new TextArea();
        m_consultArea = new TextArea("", 4, 20, TextArea.SCROLLBARS_VERTICAL_ONLY);
        
        m_editArea.addKeyListener(this);
        
        Panel pan = new Panel();
        pan.setLayout(new GridLayout(1,1));
        
        pan.add(m_editArea);
        
        add("Center", pan);
        
        pan = new Panel();
        pan.setLayout(new GridLayout(1,1));
        pan.add(m_consultArea);
        
        add("South", pan);
 
        Font ft = new Font("Courier", Font.PLAIN, 12);        
        m_editArea.setFont(ft);      
        m_consultArea.setFont(ft);              
        
        if(bOpen)
            openFile();
        else
            m_strFileName = null;
            
        m_editArea.requestFocus();
     }
    
     public void actionPerformed(ActionEvent e) 
     {
        String command = e.getActionCommand();        
           
        if (command.equals("New")) 
        {
            EditFrame edtFrame = new EditFrame(m_engine, false);
            edtFrame.show();
        }
        else if (command.equals("Open")) 
        {
            m_consultArea.setText("");
            m_editArea.setText("");

            openFile();
        } 
        else if (command.equals("Save")) 
        {
            m_consultArea.setText("");
            
			saveFileEx();
        } 
        else if (command.equals("Save as")) 
        {
            m_consultArea.setText("");

            saveFileAs();
        } 
        else if (command.equals("Consult"))// || command.equals("Reconsult")) 
        {
            if(m_editArea.getText() == "")
                return;
                
            waitCursor(true);                        
            
            consult();
            waitCursor(false);
        }
        else if (command.equals("Help me")) 
        {
            m_consultArea.setText("Type your prolog code here and consult it by F7 or File/Consult or CTRL+N.\n");
            m_consultArea.append("Then type your query in the console and press <enter>");            
        } 
        else if (command.equals("Close")) 
        {
            onDestroy();
        } 

     }    
         
    private void waitCursor(boolean bWait)
    {
        if(bWait)
            m_editArea.setCursor(new Cursor(Cursor.WAIT_CURSOR));
        else
            m_editArea.setCursor(new Cursor(Cursor.DEFAULT_CURSOR));
    }
    
    private void openFile()
    {
        String strFileName = null;
        try
        {
            FileDialog fDlg = new FileDialog(this, "Open", FileDialog.LOAD);        
            fDlg.setModal(true);
            fDlg.show();
            strFileName = fDlg.getDirectory() + fDlg.getFile();
			//System.out.println(strFileName);			
        }
        catch(SecurityException ex)
        {
            m_consultArea.append("You are running JIP from a web page. \nDue to security limitation imposed by the browser you cannot open or save file on local machine\n");
        }
        
        if(strFileName == null)
            return;
        
        try
        {
            DataInputStream ins = new DataInputStream(new FileInputStream(strFileName));            
        
            loadFile(ins);
            
            ins.close();
        }
        catch(SecurityException ex)
        {
			m_consultArea.append("\nMaybe you are running JIP from a web page. \nDue to security limitation imposed by the browser you cannot open or save file on local machine");
			return;
        }
        catch(Exception ex)
        {
            m_consultArea.append(ex.getMessage());
            m_consultArea.append("\n");                        
            return;
        }
        
        m_consultArea.append("Opened file: " + strFileName);
        
        m_strFileName = strFileName;
        
        setTitle(m_strFileName);
    }
    
    private void saveFile()
    {        
        try
        {        
            StringBufferInputStream sbins = new StringBufferInputStream(m_editArea.getText());         
            DataInputStream ins = new DataInputStream(sbins);                                 
            //DataInputStream ins = new DataInputStream(new TextAreaInputStream(m_editArea));            
            PrintStream     outs = new PrintStream(new FileOutputStream(m_strFileName));
            
            String strLine;
            while ((strLine = ins.readLine()) != null)
                outs.println(strLine);            
                
            ins.close();
            outs.close();
        }
        catch(SecurityException ex)
        {
        	m_consultArea.append("\nMaybe you are running JIP from a web page. \nDue to security limitation imposed by the browser you cannot open or save file on local machine\n");
        	return;
        }		
        catch(Exception ex)
        {
            m_consultArea.append(ex.getMessage());         
            m_consultArea.append("\n");            
            return;
        }
        
        m_consultArea.append("Saved file: " + m_strFileName + "\n");
    }

    private void saveFileAs()
    {
        String strFileName = null;
        
        try
        {            
            FileDialog fDlg = new FileDialog(this, "Save as", FileDialog.SAVE);        
            fDlg.setModal(true);
            fDlg.setFile(m_strFileName);
            fDlg.show();
			strFileName = fDlg.getDirectory() + fDlg.getFile();
        }
        catch(SecurityException ex)
        {
            m_consultArea.append("You are running JIP from a web page. \nDue to security limitation imposed by the browser you cannot open or save file on local machine\n");
        }
        
        if(strFileName == null)
            return;
            
        m_strFileName = strFileName;
        setTitle(m_strFileName);
        saveFile();                
    }

    public void keyPressed(KeyEvent e)
    {     
        if(e.getKeyCode() == KeyEvent.VK_TAB)
        {
            m_editArea.insert("   ", m_editArea.getCaretPosition());
            e.consume();
        }                        
        else if(e.getKeyCode() == KeyEvent.VK_F7)
        {
            consult();
        }                
    }
    
    public void keyReleased(KeyEvent e) {}
    public void keyTyped(KeyEvent e) {}

	private void consult()
	{
		m_consultArea.setText("");
	
		// before consulting save the file
		saveFileEx();
		
		PrintStream ous = m_engine.getOutputStream();
        m_engine.setOutputStream(new PrintStream(new TextAreaOutputStream(m_consultArea)));
		
		m_engine.consultFile(m_strFileName);
		
        m_engine.setOutputStream(ous);		
	}
	
    private void consultEditArea()
    {
        m_consultArea.setText("");
		
        DataInputStream ins = m_engine.getInputStream();
        PrintStream     ous = m_engine.getOutputStream();
        
        StringBufferInputStream sbins = new StringBufferInputStream(m_editArea.getText());       
        //m_engine.setInputStream(new DataInputStream(new TextAreaInputStream(m_editArea)));
        m_engine.setInputStream(new DataInputStream(sbins));
        m_engine.setOutputStream(new PrintStream(new TextAreaOutputStream(m_consultArea)));

        m_engine.consultInputStream();

        m_engine.setInputStream(ins);
        m_engine.setOutputStream(ous);		
    }

    private void loadFile(DataInputStream ins)
    {    
        try
        {        
            String strFile = "";
            
            String strLine;
            while ((strLine = ins.readLine()) != null)
                strFile += strLine + "\n";
                
            ins.close();
            
            m_editArea.setText(strFile);                
            m_editArea.requestFocus();
       }
       catch(SecurityException ex)
       {
       		m_consultArea.append("\nMaybe you are running JIP from a web page. \nDue to security limitation imposed by the browser you cannot open or save file on local machine\n");
       		return;
       }	   
       catch(Exception ex)
       {
            m_consultArea.append(ex.getMessage());
            m_consultArea.append("\n");            
            return;
       }
    }
	
	public void saveFileEx()
	{
		if(m_strFileName == null)
		    saveFileAs();
		else
		    saveFile();	
	}
}