package JIP;

import JIP.engine.*;
import java.util.Enumeration;
import java.io.*;
import java.util.StringTokenizer;

public class TextClausesEnumeration extends JIPClausesEnumeration
{  
    private LineNumberReader m_reader;    
    private JIPClause        m_curClause;
    private boolean          m_bUsed;
    
    public TextClausesEnumeration(TextClausesDatabase db)
    {
        super(db);
        
        try
        {
            m_reader = new LineNumberReader(new FileReader(db.getFileName()));
        }
        catch(IOException ex)
        {
            throw new RuntimeException(ex.getMessage());
        }
                      
        updateCurClause();
    }
     
    public boolean hasMoreElements()
    {
        if(m_bUsed)
            updateCurClause();
               
        if(m_curClause == null)
        {
            finalize();
            return false;
        }
            
        return true;
    }   
    
    public JIPClause nextClause()
    {
        m_bUsed = true;
        return m_curClause;        
    }
    
    private void updateCurClause()
    {
        m_bUsed = false;
        
        String strLine;
        try
        {
            // read next line
            strLine = m_reader.readLine();

            if(strLine == null)
            {
                m_curClause = null;
                m_reader.close();
                return;
            }            
        }
        catch(IOException ex)
        {            
            throw new RuntimeException(ex.getMessage());
        }
     
        //separate parameters by StringTokenizer
        StringTokenizer stk = new StringTokenizer(strLine, "#");
        
        String  strTerm;
        JIPTerm term;
        JIPList list = null;
        
        int i = 0;
        while (stk.hasMoreTokens())
        {
            i++;
            
            // extract next term
            strTerm = stk.nextToken();
            
            try
            {
                //parse the term extracted
                term    = JIPTerm.parseTerm(strTerm);
            }
            catch(JIPSyntaxErrorException ex)
            {
                finalize();
                throw new RuntimeException(ex.getMessage());
            }
            
            //add to list
            list = JIPList.getNewList(term, list);            
        }       
        
        if(i != getArity())
        {
            finalize();
            throw new RuntimeException("The arity of the extern predicate doesn't match with the expected one");
        }

        // reverse the list because was constructed from tail
        list = list.reverse();         
        
        // make functor        
        JIPFunctor func = JIPFunctor.getNewFunctor(getFunctorName(), list);
        
        // make clause
        m_curClause = JIPClause.getNewClause(func, null);
    }
       
    public void finalize()
    {
        try
        {
            m_reader.close();
        }
        catch(IOException ex){};
    }
}