package JIP;

import Ugosoft.awt.XDialog;
import Ugosoft.io.*;
import java.awt.*;
import java.io.*;
import java.net.*;
import java.awt.event.*;
import JIP.engine.*;
    
class TraceDialog extends XDialog implements ActionListener, JIPTraceListener, JIPEventListener, JIPErrorListener
{
    private static final int WIDTH  = 400;
    private static final int HEIGHT = 200;
    
    private Button              m_btnSkip;
    private Button              m_btnStep;
    private Button              m_btnAbort;
    private TextArea            m_traceArea;

    private JIPTraceEvent   m_traceEvent;
    
    private PrintStream     m_outs             = null;

    public TraceDialog(Frame parent)
    {
        super(parent, "JIP - Trace Window", false);
                           
        Dimension ds = Toolkit.getDefaultToolkit().getScreenSize();
        int nWidth = (int) ((double)ds.width * (5.0/10.0));

        Rectangle d = parent.getBounds();
            
        int nTop  = d.y + (d.height - HEIGHT) / 2;
        int nLeft = d.x + (d.width  - nWidth)  / 2;        
        setBounds(nLeft, nTop, nWidth, HEIGHT);

        setResizable(true);

        m_btnStep     = new Button("Step");
        m_btnAbort    = new Button("Abort");
        m_btnSkip     = new Button("Skip");        
        m_traceArea   = new TextArea();        
        
        m_traceArea.setFont(new Font("Courier", Font.PLAIN, 12));
      
        m_btnStep.addActionListener(this);
        m_btnAbort.addActionListener(this);
        m_btnSkip.addActionListener(this);
        
        m_btnStep.setEnabled(false);
        m_btnSkip.setEnabled(false);
        m_btnAbort.setEnabled(false);
        
        setLayout(new BorderLayout());
       
        Panel panBtn = new Panel();
          
        GridBagLayout layout   = new GridBagLayout();
        GridBagConstraints gbc = new GridBagConstraints();
    
        panBtn.setLayout(layout);
        
        // Repositioning components on main panel
        gbc.fill = GridBagConstraints.HORIZONTAL;
        gbc.anchor = GridBagConstraints.CENTER;
        gbc.gridx = GridBagConstraints.RELATIVE;
        gbc.insets = new Insets(3, 3, 3, 3);    
        
        layout.setConstraints(m_btnStep, gbc);       
        panBtn.add(m_btnStep);
        layout.setConstraints(m_btnSkip, gbc);       
        panBtn.add(m_btnSkip);
        layout.setConstraints(m_btnAbort, gbc);       
        panBtn.add(m_btnAbort);              
        
        add("North", panBtn);
        add("Center", m_traceArea);                   
        
        m_outs = new TextAreaPrintStream(new TextAreaOutputStream(m_traceArea));                   
     }
    
     public void actionPerformed(ActionEvent evt) 
     {
        if(evt.getSource() == m_btnStep)
        {
            waitCursor(true);
            m_traceEvent.step();          
            m_btnStep.setEnabled(false);       
        }
        else if(evt.getSource() == m_btnSkip)
        {
            waitCursor(true);
            m_traceEvent.skip();
            m_btnSkip.setEnabled(false);            
        }
        else if (evt.getSource() == m_btnAbort)
        {    
            waitCursor(true);
            m_traceEvent.abort();            
            hide();
        }
     }    
         
    private void waitCursor(boolean bWait)
    {
        if(bWait)
            setCursor(new Cursor(Cursor.WAIT_CURSOR));
        else
            setCursor(new Cursor(Cursor.DEFAULT_CURSOR));
    }
    
    public void foundNotified(JIPTraceEvent e)
    {
        m_traceEvent = e;
        m_btnStep.setEnabled(true);
        m_btnSkip.setEnabled(false);
        m_btnAbort.setEnabled(true);
        
        StringBuffer strSpace = new StringBuffer((int)e.getHeight());
        for (int i = 0; i < e.getHeight(); i++)
        {
            strSpace.append(".");
        }     
        
        m_outs.print(e.getHeight());   
        m_outs.print("> ");   

        m_outs.print(strSpace);         
        m_outs.print("Found:");   
        
        m_outs.println(e.getTerm());           

        waitCursor(false);
    }

    public void callNotified(JIPTraceEvent e)
    {             
        m_traceEvent = e;
        m_btnStep.setEnabled(true);
        m_btnSkip.setEnabled(false);
        m_btnAbort.setEnabled(true);

        StringBuffer strSpace = new StringBuffer(e.getHeight());
        for (int i = 0; i < e.getHeight(); i++)
        {
            strSpace.append(".");
        }     
        
        m_outs.print(e.getHeight());   
        m_outs.print("> ");   
        m_outs.print(strSpace);   
        m_outs.print("Call:");   
        m_outs.println(e.getTerm());

		//System.out.println(e.getQueryHandle());
        waitCursor(false);
    }

    public void boundNotified(JIPTraceEvent e)
    {
        m_traceEvent = e;
        m_btnStep.setEnabled(true);
        m_btnSkip.setEnabled(true);
        m_btnAbort.setEnabled(true);
        
        StringBuffer strSpace = new StringBuffer(e.getHeight());
        for (int i = 0; i < e.getHeight(); i++)
        {
            strSpace.append(".");
        }     
        
        m_outs.print(e.getHeight());   
        m_outs.print("> ");   
        m_outs.print(strSpace);   
        m_outs.print("Bound to:");           
        m_outs.println(e.getTerm());   

        waitCursor(false);
    }

    public void redoNotified(JIPTraceEvent e)
    {
        m_traceEvent = e;
        m_btnStep.setEnabled(true);
        m_btnSkip.setEnabled(false);
        m_btnAbort.setEnabled(true);

        StringBuffer strSpace = new StringBuffer(e.getHeight());
        for (int i = 0; i < e.getHeight(); i++)
        {
            strSpace.append(".");
        }     
        
        m_outs.print(e.getHeight());   
        m_outs.print("> ");   
        m_outs.print(strSpace);   
        m_outs.print("Redo:");   
        m_outs.println(e.getTerm());   

        waitCursor(false);
    }
    
        // A Start event occurred
    public void openNotified(JIPEvent e)
    {}
    
    public void moreNotified(JIPEvent e)
    {}
    
    // A solution event occurred    
    public void solutionNotified(JIPEvent e)
    {         
        if(!isVisible())
            return;
            
        // Show Solution 
        m_outs.println("Yes");
        
        JIPTerm term = e.getTerm();
        JIPVariable[] vars = term.getJIPVariables();
        
        for(int i = 0; i < vars.length; i++)
        {
            m_outs.println(vars[i]);
        }                

        waitCursor(false);                     
    }
    
    // A Term has been notified with notify/2
    public void termNotified(JIPEvent e)
    {}
    
    // The end has been reached because there wasn't more solutions
    public void endNotified(JIPEvent e)
    {
	    if(!isVisible())
        	return;
	
        m_outs.println("No");    
         
        waitCursor(false);     
    }
    
    public void closeNotified(JIPEvent e)
    {
		hide();
	}

    // An error (exception) has been raised up by prolog engine
    public void errorNotified(JIPErrorEvent e)
    {
		if(!isVisible())
        	return;
			
        m_outs.println("Error:");
        m_outs.println(e.getError());
              
        waitCursor(false);                        
    }
	
	public void windowClosing(WindowEvent e)
    {
      	// do nothing
    } 
}